from importlib import import_module

import os
from flask.ext.script import Command, Option


class CICommand(Command):
    """
    Perform CI operations
    :param settings: settings
    """

    help = description = 'Perform CI operations.'

    def __init__(self, settings):
        super(Command, self).__init__()
        self.tasks_cls = [import_module(module_name).Reporter for module_name in self.get_task_list(settings)]
        self.tasks = [task_cls() for task_cls in self.tasks_cls]
        self.settings = settings

    @staticmethod
    def get_task_list(settings):
        return getattr(settings, 'JENKINS_TASKS', ())

    def get_options(self):
        options = [
            Option('-o', '--output-dir', dest='output_dir', default='reports'),
            Option('-v', '--verbose', action='store_true', dest='verbose', default=False)
        ]

        for task in self.tasks:
            if hasattr(task, 'add_arguments'):
                options = options + task.get_arguments()

        return options

    def __call__(self, *args, **kwargs):
        output_dir = kwargs['output_dir']
        verbose = kwargs['verbose']

        if not os.path.exists(output_dir):
            if verbose:
                print 'Creating output directory...'
            os.makedirs(output_dir)

        for task in self.tasks:
            if verbose:
                print('Executing {0}...'.format(task.__module__))
            task.run(self.settings, **kwargs)

        if verbose:
            print('Done')



import os.path
import pep8
from flask.ext.script import Option


class Reporter(object):
    @staticmethod
    def get_arguments():
        return [
            Option('--pep8-rcfile', dest='pep8-rcfile')
        ]

    @staticmethod
    def get_config_path(settings, options):
        if options.get('pep8-rcfile', None):
            return options['pep8-rcfile']

        rcfile = getattr(settings, 'PEP8_RCFILE', '')
        if os.path.exists(rcfile):
            return rcfile

        # use built-in
        root_dir = os.path.normpath(os.path.dirname(__file__))
        return os.path.join(root_dir, 'pep8.rc')

    def run(self, settings, **options):
        output = open(os.path.join(options['output_dir'], 'pep8.report'), 'w')

        class JenkinsReport(pep8.BaseReport):
            def error(self, line_number, offset, text, check):
                code = super(JenkinsReport, self).error(line_number, offset, text, check)
                if code:
                    source_line = self.line_offset + line_number
                    output.write('%s:%s:%s: %s\n' % (self.filename, source_line, offset + 1, text))

        pep8_options = {}
        config_file = self.get_config_path(settings, options)
        pep8_options['config_file'] = config_file

        pep8style = pep8.StyleGuide(
            parse_argv=False,
            reporter=JenkinsReport,
            **pep8_options)

        pep8style.options.report.start()
        for location in getattr(settings, 'PROJECT_APPS', []):
            pep8style.input_dir(os.path.relpath(location))
        pep8style.options.report.stop()

        output.close()



import os
from flask.ext.script import Option
from pylint import lint
from pylint.reporters.text import TextReporter


class ParseableTextReporter(TextReporter):
    """
    Outputs messages in a form recognized by jenkins
    <filename>:<linenum>:<msg>
    """
    name = 'parseable'
    line_format = '{path}:{line}: [{msg_id}({symbol}), {obj}] {msg}'


class Reporter(object):
    @staticmethod
    def get_arguments():
        return [
            Option('--pylint-rcfile', dest='pylint-rcfile')
        ]

    @staticmethod
    def get_config_path(settings, options):
        if options.get('pylint_rcfile', None):
            return options['pylint-rcfile']

        rcfile = getattr(settings, 'PYLINT_RCFILE', 'pylint.rc')
        if os.path.exists(rcfile):
            return rcfile

        # use built-in
        root_dir = os.path.normpath(os.path.dirname(__file__))
        return os.path.join(root_dir, 'pylint.rc')

    def run(self, settings, **options):
        output = open(os.path.join(options['output_dir'], 'pylint.report'), 'w')

        args = list()
        args.append("--rcfile=%s" % self.get_config_path(settings, options))
        args += getattr(settings, 'PROJECT_APPS', [])

        lint.Run(args, reporter=ParseableTextReporter(output=output), exit=False)

        output.close()



"""
Flask-CI
--------

Provide Continuous Integration support operations for Flask apps.
"""
from setuptools import setup

version = '0.3.19'
packages = ['flask_ci', 'flask_ci.management', 'flask_ci.tasks', 'flask_ci.management.commands']
install_requires = ['Flask-Script', 'Pep8', 'Pylint']
flask_ci_pkg_data = ['tasks/pylint.rc', 'tasks/pep8.rc']

setup(
    name='Flask-CI',
    version=version,
    description='Continuous Integration support for Flask',
    url='https://github.com/vicenteneto/flask-ci',
    author='Vicente Neto',
    author_email='sneto.vicente@gmail.com',
    license='MIT',
    packages=packages,
    install_requires=install_requires,
    package_data={'flask_ci': flask_ci_pkg_data},
    zip_safe=False,
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Web Environment',
        'Framework :: Flask',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Software Development :: Testing'
    ]
)
